#Libraries
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import griddata, NearestNDInterpolator
from numpy import uint32, uint64, float64, float32, linspace, array, logspace, sin, cos, pi, arange
import matplotlib.colors as mc
import timeit
import h5py

def read_snapshot(filename):
    data = h5py.File(filename + '.hdf5',"r")
    #f = open(filename, mode='rb')
    #print ("Loading file %s" %filename)

    with h5py.File(filename + '.hdf5', 'r') as f:
        header = dict(f['Header'].attrs.items())
    #print("Header :: ",header)
    return data, header


#=====================================================
start = timeit.default_timer()
#========================================================
########### GENERAL STUFF  ##########

#internal arepo units in cgs
arepoLength = 3.0856e21 #[cm]   ~1 kParsec
arepoMass   = 1.991e43    #[g]    ~10^10 Sun Mass
arepoVel    = 1.0e5        #[cm/s] ~1km/s

arepoTime    = arepoLength/arepoVel  
arepoDensity = arepoMass/arepoLength/arepoLength/arepoLength  # d = m/V = M/L^3
arepoEnergy  = arepoMass*arepoVel*arepoVel                      # E = mv^2
arepoColumnDensity = arepoMass/arepoLength/arepoLength

Floattype = np.float64
#========================================================
#File To Read
number = 10
base = './snaps/'
sim = 'snap_010'

filenum = str(number).zfill(3)

#File Name (without '.hdf5')
f = base + sim #+ filenum

print('Reading snapshot :: ', f)
#data, header = rsnap_binary.read_snapshot(f)
#data, header = rsnap_hdf5.read_snapshot(f)
data, header = read_snapshot(f)

# nparts = header['num_particles']
nparts = header['NumPart_Total']

#h5dump -n data_samples/IsolatedGalaxy-MW-type/snap_000.hdf5
rho = np.array(data["PartType0"]["Density"],dtype=Floattype)
rho = rho * arepoDensity / 1.27 / 1.6735325e-24
Poss = np.array(data["PartType0"]["Coordinates"],dtype=Floattype)

x, y, z = Poss[:,0],Poss[:,1],Poss[:,2]

center = np.array([np.average(x),np.average(y),np.average(z)])
print("Center=",center)

x,y,z = x-center[0], y-center[1],z-center[2]

points = np.vstack((x,y,z)).T
rhof = NearestNDInterpolator(points,rho)

xmin, xmax, ymin, ymax, zcut, ngrid = -100., 100., -100., 100., -3, 2001

# produce horizontal slices at different heights
xgrid, ygrid, zgrid = linspace(xmin,xmax,ngrid), linspace(ymin,ymax,ngrid), linspace(zcut,zcut,1)
XGRID,YGRID,ZGRID = np.meshgrid(xgrid,ygrid,zgrid)
RHO = rhof(XGRID,YGRID,ZGRID)

# plot horizontal slices
#Colormaps: https://matplotlib.org/stable/tutorials/colors/colormaps.html
nslices = zgrid.size
fig, axarr = plt.subplots(1,1,figsize=(12,11))

extent = (xmin, xmax, ymin, ymax)

im = plt.imshow(RHO[:,:,0],origin='lower',norm=mc.LogNorm(),extent=extent,cmap='plasma') #'Greys', 'plasma', 'gist_stern'

plt.xlabel(r'$x\, ({\rm kpc})$',fontsize=25)
plt.ylabel(r'$y\, ({\rm kpc})$',fontsize=25)
plt.xlim(-50,50)
plt.ylim(-50,50)

cbar = fig.colorbar(im)
cbar.set_label('Number Density [cm$-3$]',size=25)

fig.savefig('slicesz3.png',bbox_inches='tight')